//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// Definitions for the "dye" manipulator, that allows to easily use
// ANSI color escape sequences in C++ programs.
//
// BEWARE: the package does not test that the output stream is linked
//         to an ANSI-compliant output device.
// -------------------------------------------------------------------
// $Id: dye.hh,v 1.3 2005/06/14 08:57:34 loreti Exp $
// -------------------------------------------------------------------

#ifndef DYE_HH_
#define DYE_HH_

#include <ios>
#include <ostream>
#include <string>
#include <cstdarg>

#ifdef DEBUG
  #include <iostream>
#endif // DEBUG

// The class "dye", that defines:

// - a manipulator, that inserts in an output stream the ANSI escape
//   sequence that corresponds to a given change of character style;
// - an enumeration, that defines readable shortcuts for every change
//   of style that may be required;
// - Two constructors: that build internally the above mentioned ANSI
//   sequence, starting or from the request for a single required
//   change (identified from a single formal parameter); or from the
//   request for several changes, identified by a sequence of several
//   parameters terminated by the "sentinel" value dye::end.
// - the destructor (dummy).

class dye {
public:
  enum code {                   // Shortcuts for color codes etc.
    reset, bold, underline, blink, reverse,
    black, red, green, yellow, blue, magenta, cyan, white,
    onblack, onred, ongreen, onyellow, onblue, onmagenta,
    oncyan, onwhite, end
  };

  explicit dye(code c) : _seq("\033[") {
    seq_append(c);  _seq.push_back('m');
#ifdef DEBUG
    seq_out();
#endif // DEBUG
  }
  explicit dye(code, code, ...);
  virtual ~dye() {}

private:
  std::string _seq;             // ANSI escape sequence

  void seq_append(code);        // Internal
#ifdef DEBUG
  void seq_out(std::ostream & o = std::cerr); // Internal
#endif // DEBUG

  // For what concerns the code of the manipulator (that follows
  // below), see e.g. the book "Standard C++ IOStreams and Locales" by
  // Angelika Langer and Klaus Kreft (published by Addison-Wesley in
  // 1999, ISBN 0-201-18395-1).

  template <typename charT, typename Traits>
  friend std::basic_ostream<charT, Traits> &
  operator << (
    std::basic_ostream<charT, Traits> & o,
    const dye & c
  ) {
    o << c._seq;
    return o;
  }
};

#endif // DYE_HH_
